@@ -1,7 +1,7 @@
 # Authors: David Goodger
 # Contact: goodger@python.org
-# Revision: $Revision: 1.50 $
-# Date: $Date: 2004/07/25 13:36:33 $
+# Revision: $Revision: 4208 $
+# Date: $Date: 2005-12-14 03:12:00 +0100 (Wed, 14 Dec 2005) $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -18,9 +18,12 @@ __docformat__ = 'reStructuredText'
 
 import sys
 import pprint
-from docutils import __version__, SettingsSpec
+from types import StringType
+from docutils import __version__, __version_details__, SettingsSpec
 from docutils import frontend, io, utils, readers, writers
 from docutils.frontend import OptionParser
+from docutils.transforms import Transformer
+import docutils.readers.doctree
 
 
 class Publisher:
@@ -39,6 +42,9 @@ class Publisher:
         a component name (`set_reader` sets the parser as well).
         """
 
+        self.document = None
+        """The document tree (`docutils.nodes` objects)."""
+
         self.reader = reader
         """A `docutils.readers.Reader` instance."""
 
@@ -48,6 +54,11 @@ class Publisher:
         self.writer = writer
         """A `docutils.writers.Writer` instance."""
 
+        for component in 'reader', 'parser', 'writer':
+            assert not isinstance(getattr(self, component), StringType), \
+                   ('passed string as "%s" parameter; use "%s_name" instead'
+                    % (getattr(self, component), component, component))
+
         self.source = source
         """The source of input data, a `docutils.io.Input` instance."""
 
@@ -168,11 +179,11 @@ class Publisher:
             encoding=self.settings.output_encoding,
             error_handler=self.settings.output_encoding_error_handler)
 
-    def apply_transforms(self, document):
-        document.transformer.populate_from_components(
+    def apply_transforms(self):
+        self.document.transformer.populate_from_components(
             (self.source, self.reader, self.reader.parser, self.writer,
              self.destination))
-        document.transformer.apply_transforms()
+        self.document.transformer.apply_transforms()
 
     def publish(self, argv=None, usage=None, description=None,
                 settings_spec=None, settings_overrides=None,
@@ -182,46 +193,60 @@ class Publisher:
         already set), run `self.reader` and then `self.writer`.  Return
         `self.writer`'s output.
         """
-        if self.settings is None:
-            self.process_command_line(
-                argv, usage, description, settings_spec, config_section,
-                **(settings_overrides or {}))
-        self.set_io()
         exit = None
-        document = None
         try:
-            document = self.reader.read(self.source, self.parser,
-                                        self.settings)
-            self.apply_transforms(document)
-            output = self.writer.write(document, self.destination)
+            if self.settings is None:
+                self.process_command_line(
+                    argv, usage, description, settings_spec, config_section,
+                    **(settings_overrides or {}))
+            self.set_io()
+            self.document = self.reader.read(self.source, self.parser,
+                                             self.settings)
+            self.apply_transforms()
+            output = self.writer.write(self.document, self.destination)
             self.writer.assemble_parts()
+        except SystemExit, error:
+            exit = 1
+            exit_status = error.code
         except Exception, error:
-            if self.settings.traceback: # propagate exceptions?
+            if not self.settings:       # exception too early to report nicely
+                raise
+            if self.settings.traceback: # Propagate exceptions?
+                self.debugging_dumps()
                 raise
             self.report_Exception(error)
             exit = 1
-        self.debugging_dumps(document)
-        if (enable_exit_status and document
-            and (document.reporter.max_level
+            exit_status = 1
+        self.debugging_dumps()
+        if (enable_exit_status and self.document
+            and (self.document.reporter.max_level
                  >= self.settings.exit_status_level)):
-            sys.exit(document.reporter.max_level + 10)
+            sys.exit(self.document.reporter.max_level + 10)
         elif exit:
-            sys.exit(1)
+            sys.exit(exit_status)
         return output
 
-    def debugging_dumps(self, document):
+    def debugging_dumps(self):
+        if not self.document:
+            return
         if self.settings.dump_settings:
             print >>sys.stderr, '\n::: Runtime settings:'
             print >>sys.stderr, pprint.pformat(self.settings.__dict__)
-        if self.settings.dump_internals and document:
+        if self.settings.dump_internals:
             print >>sys.stderr, '\n::: Document internals:'
-            print >>sys.stderr, pprint.pformat(document.__dict__)
-        if self.settings.dump_transforms and document:
+            print >>sys.stderr, pprint.pformat(self.document.__dict__)
+        if self.settings.dump_transforms:
             print >>sys.stderr, '\n::: Transforms applied:'
-            print >>sys.stderr, pprint.pformat(document.transformer.applied)
-        if self.settings.dump_pseudo_xml and document:
+            print >>sys.stderr, (' (priority, transform class, '
+                                 'pending node details, keyword args)')
+            print >>sys.stderr, pprint.pformat(
+                [(priority, '%s.%s' % (xclass.__module__, xclass.__name__),
+                  pending and pending.details, kwargs)
+                 for priority, xclass, pending, kwargs
+                 in self.document.transformer.applied])
+        if self.settings.dump_pseudo_xml:
             print >>sys.stderr, '\n::: Pseudo-XML:'
-            print >>sys.stderr, document.pformat().encode(
+            print >>sys.stderr, self.document.pformat().encode(
                 'raw_unicode_escape')
 
     def report_Exception(self, error):
@@ -234,9 +259,10 @@ class Publisher:
             print >>sys.stderr, ("""\
 Exiting due to error.  Use "--traceback" to diagnose.
 Please report errors to <docutils-users@lists.sf.net>.
-Include "--traceback" output, Docutils version (%s),
+Include "--traceback" output, Docutils version (%s [%s]),
 Python version (%s), your OS type & version, and the
-command line used.""" % (__version__, sys.version.split()[0]))
+command line used.""" % (__version__, __version_details__,
+                         sys.version.split()[0]))
 
     def report_SystemMessage(self, error):
         print >>sys.stderr, ('Exiting due to level-%s (%s) system message.'
@@ -371,7 +397,8 @@ def publish_string(source, source_path=None, destination_path=None,
         enable_exit_status=enable_exit_status)
     return output
 
-def publish_parts(source, source_path=None, destination_path=None,
+def publish_parts(source, source_path=None, source_class=io.StringInput,
+                  destination_path=None,
                   reader=None, reader_name='standalone',
                   parser=None, parser_name='restructuredtext',
                   writer=None, writer_name='pseudoxml',
@@ -387,12 +414,12 @@ def publish_parts(source, source_path=None, destination_path=None,
     the desired encoding.  Set it to 'unicode' for unencoded Unicode string
     input.  Here's how::
 
-        publish_string(..., settings_overrides={'input_encoding': 'unicode'})
+        publish_parts(..., settings_overrides={'input_encoding': 'unicode'})
 
     Parameters: see `publish_programmatically`.
     """
     output, pub = publish_programmatically(
-        source_class=io.StringInput, source=source, source_path=source_path,
+        source=source, source_path=source_path, source_class=source_class,
         destination_class=io.StringOutput,
         destination=None, destination_path=destination_path,
         reader=reader, reader_name=reader_name,
@@ -404,21 +431,92 @@ def publish_parts(source, source_path=None, destination_path=None,
         enable_exit_status=enable_exit_status)
     return pub.writer.parts
 
+def publish_doctree(source, source_path=None,
+                    source_class=io.StringInput,
+                    reader=None, reader_name='standalone',
+                    parser=None, parser_name='restructuredtext',
+                    settings=None, settings_spec=None,
+                    settings_overrides=None, config_section=None,
+                    enable_exit_status=None):
+    """
+    Set up & run a `Publisher` for programmatic use with string I/O.
+    Return the document tree.
+
+    For encoded string input, be sure to set the 'input_encoding' setting to
+    the desired encoding.  Set it to 'unicode' for unencoded Unicode string
+    input.  Here's one way::
+
+        publish_doctree(..., settings_overrides={'input_encoding': 'unicode'})
+
+    Parameters: see `publish_programmatically`.
+    """
+    pub = Publisher(reader=reader, parser=parser, writer=None,
+                    settings=settings,
+                    source_class=source_class,
+                    destination_class=io.NullOutput)
+    pub.set_components(reader_name, parser_name, 'null')
+    pub.process_programmatic_settings(
+        settings_spec, settings_overrides, config_section)
+    pub.set_source(source, source_path)
+    pub.set_destination(None, None)
+    output = pub.publish(enable_exit_status=enable_exit_status)
+    return pub.document
+
+def publish_from_doctree(document, destination_path=None,
+                         writer=None, writer_name='pseudoxml',
+                         settings=None, settings_spec=None,
+                         settings_overrides=None, config_section=None,
+                         enable_exit_status=None):
+    """
+    Set up & run a `Publisher` to render from an existing document tree data
+    structure, for programmatic use with string I/O.  Return a pair of encoded
+    string output and document parts.
+
+    Note that document.settings is overridden; if you want to use the settings
+    of the original `document`, pass settings=document.settings.
+
+    Also, new document.transformer and document.reporter objects are
+    generated.
+
+    For encoded string output, be sure to set the 'output_encoding' setting to
+    the desired encoding.  Set it to 'unicode' for unencoded Unicode string
+    output.  Here's one way::
+
+        publish_from_doctree(
+            ..., settings_overrides={'output_encoding': 'unicode'})
+
+    Parameters: `document` is a `docutils.nodes.document` object, an existing
+    document tree.
+
+    Other parameters: see `publish_programmatically`.
+    """
+    reader = docutils.readers.doctree.Reader(parser_name='null')
+    pub = Publisher(reader, None, writer,
+                    source=io.DocTreeInput(document),
+                    destination_class=io.StringOutput, settings=settings)
+    if not writer and writer_name:
+        pub.set_writer(writer_name)
+    pub.process_programmatic_settings(
+        settings_spec, settings_overrides, config_section)
+    pub.set_destination(None, destination_path)
+    return pub.publish(enable_exit_status=enable_exit_status)
+
 def publish_programmatically(source_class, source, source_path,
-                            destination_class, destination, destination_path,
-                            reader, reader_name,
-                            parser, parser_name,
-                            writer, writer_name,
-                            settings, settings_spec,
-                            settings_overrides, config_section,
-                            enable_exit_status):
+                             destination_class, destination, destination_path,
+                             reader, reader_name,
+                             parser, parser_name,
+                             writer, writer_name,
+                             settings, settings_spec,
+                             settings_overrides, config_section,
+                             enable_exit_status):
     """
     Set up & run a `Publisher` for custom programmatic use.  Return the
     encoded string output and the Publisher object.
 
     Applications should not need to call this function directly.  If it does
     seem to be necessary to call this function directly, please write to the
-    docutils-develop@lists.sourceforge.net mailing list.
+    Docutils-develop mailing list
+    <http://docutils.sf.net/docs/user/mailing-lists.html#docutils-develop>.
 
     Parameters:
 
@@ -427,14 +525,15 @@ def publish_programmatically(source_class, source, source_path,
 
     * `source`: Type depends on `source_class`:
 
-      - `io.FileInput`: Either a file-like object (must have 'read' and
-        'close' methods), or ``None`` (`source_path` is opened).  If neither
-        `source` nor `source_path` are supplied, `sys.stdin` is used.
+      - If `source_class` is `io.FileInput`: Either a file-like object
+        (must have 'read' and 'close' methods), or ``None``
+        (`source_path` is opened).  If neither `source` nor
+        `source_path` are supplied, `sys.stdin` is used.
 
-      - `io.StringInput` **required**: The input string, either an encoded
-        8-bit string (set the 'input_encoding' setting to the correct
-        encoding) or a Unicode string (set the 'input_encoding' setting to
-        'unicode').
+      - If `source_class` is `io.StringInput` **required**: The input
+        string, either an encoded 8-bit string (set the
+        'input_encoding' setting to the correct encoding) or a Unicode
+        string (set the 'input_encoding' setting to 'unicode').
 
     * `source_path`: Type depends on `source_class`:
 
